/*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox
   \\    /   O peration     |
    \\  /    A nd           | Copyright held by original author
     \\/     M anipulation  |
-------------------------------------------------------------------------------
License
    This file is part of OpenFOAM.

    OpenFOAM is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
    Free Software Foundation; either version 2 of the License, or (at your
    option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM; if not, write to the Free Software Foundation,
    Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

\*---------------------------------------------------------------------------*/

#include "floatingBody.H"
#include "fixedValueTetPolyPatchFields.H"

#include "incompressible/transportModel/transportModel.H"
#include "incompressible/turbulenceModel/turbulenceModel.H"


// * * * * * * * * * * * * * Private Member Functions  * * * * * * * * * * * //


Foam::dimensionedVector Foam::floatingBody::readGravity(const fvMesh& mesh)
{
    Info << "\nReading environmentalProperties" << endl;

    IOdictionary environmentalProperties
    (
        IOobject
        (
            "environmentalProperties",
            mesh.time().constant(),
            mesh,
            IOobject::MUST_READ,
            IOobject::NO_WRITE
        )
    );

    dimensionedVector g(environmentalProperties.lookup("g"));

    return g;
}


void Foam::floatingBody::check() const
{
    if (!hull_.active())
    {
        FatalErrorIn("void floatingBody::check() const")
            << "Hull patch named " << hull_.name()
            << " not found.  Available patches: "
            << mesh_.boundaryMesh().names()
            << abort(FatalError);
    }
}


Foam::vector Foam::floatingBody::hullForce() const
{
    const turbulenceModel& turbulence =
        mesh_.lookupObject<turbulenceModel>("turbulenceProperties");

    volScalarField nuEff = turbulence.nuEff();

    const volScalarField& p = mesh_.lookupObject<volScalarField>("p");
    const volVectorField& U = mesh_.lookupObject<volVectorField>("U");

    // Get index of current patch
    const label curPatch = hull_.index();

    vector pressureForce =
        gSum
        (
            p.boundaryField()[curPatch]*
            mesh_.Sf().boundaryField()[curPatch]
        );

    vector viscousForce =
        gSum
        (
            -nuEff.boundaryField()[curPatch]*
            U.boundaryField()[curPatch].snGrad()*
            mesh_.magSf().boundaryField()[curPatch]
        );

    vector totalForce = pressureForce + viscousForce;

    Info<< "Pressure force = " << pressureForce << nl
        << "Viscous force = " << viscousForce << nl
        << "Total force = " << totalForce << endl;

    return totalForce;
}


Foam::vector Foam::floatingBody::hullMoment() const
{
    const turbulenceModel& turbulence =
        mesh_.lookupObject<turbulenceModel>("turbulenceProperties");

    volScalarField nuEff = turbulence.nuEff();

    const volScalarField& p = mesh_.lookupObject<volScalarField>("p");
    const volVectorField& U = mesh_.lookupObject<volVectorField>("U");

    // Get index of current patch
    const label curPatch = hull_.index();

    vector pressureMoment =
        gSum
        (
            (mesh_.Cf().boundaryField()[curPatch] - sixDOF_.X().value())
          ^ (
                p.boundaryField()[curPatch]*
                mesh_.Sf().boundaryField()[curPatch]
            )
        );

    vector viscousMoment =
        gSum
        (
            (mesh_.Cf().boundaryField()[curPatch] - sixDOF_.X().value())
          ^ (
                -nuEff.boundaryField()[curPatch]*
                U.boundaryField()[curPatch].snGrad()*
                mesh_.magSf().boundaryField()[curPatch]
            )
        );

    vector totalMoment = pressureMoment + viscousMoment;

    Info<< "Pressure moment = " << pressureMoment << nl
        << "Viscous moment = " << viscousMoment << nl
        << "Total moment = " << totalMoment << endl;

    return totalMoment;
}


// * * * * * * * * * * * * * * * * Constructors  * * * * * * * * * * * * * * //

// Construct from components
Foam::floatingBody::floatingBody
(
    const word& name,
    const fvMesh& mesh,
    const word& hullName,
    const word& bargeSliderName,
    const word& fixedSliderName,
    const word& odeSolverName,
    const scalar epsilon
)
:
    name_(name),
    mesh_(mesh),
    hull_(hullName, mesh_.boundaryMesh()),
    bargeSlider_(bargeSliderName, mesh_.boundaryMesh()),
    fixedSlider_(fixedSliderName, mesh_.boundaryMesh()),
    sixDOF_
    (
        IOobject
        (
            name,
            mesh_.time().timeName(),
            mesh_,
            IOobject::MUST_READ,
            IOobject::AUTO_WRITE
        )
    ),
    solver_
    (
        ODESolver::New
        (
            odeSolverName,
            sixDOF_
        )
    ),
    epsilon_(epsilon),
    g_(readGravity(mesh)),
    curTimeIndex_(-1),
    oldForce_(vector::zero),
    oldMoment_(vector::zero),
    of_(mesh_.time().path()/name_ + ".dat", ios_base::out|ios_base::app)
{
    check();
}


// Construct from dictionary
Foam::floatingBody::floatingBody
(
    const word& name,
    const fvMesh& mesh,
    const dictionary& dict
)
:
    name_(name),
    mesh_(mesh),
    hull_(word(dict.lookup("hull")), mesh_.boundaryMesh()),
    bargeSlider_(word(dict.lookup("bargeSlider")), mesh_.boundaryMesh()),
    fixedSlider_(word(dict.lookup("fixedSlider")), mesh_.boundaryMesh()),
    sixDOF_
    (
        IOobject
        (
            name,
            mesh_.time().timeName(),
            mesh_,
            IOobject::MUST_READ,
            IOobject::AUTO_WRITE
        )
    ),
    solver_
    (
        ODESolver::New
        (
            dict.lookup("odeSolver"),
            sixDOF_
        )
    ),
    epsilon_(readScalar(dict.lookup("epsilon"))),
    g_(readGravity(mesh)),
    curTimeIndex_(-1),
    oldForce_(vector::zero),
    oldMoment_(vector::zero),
    // Dealing with writing a motion file in parallel.  HJ, 14/Aug/2008
    of_(mesh_.time().caseConstant()/".."/name_ + ".dat")
{
    check();
}


// * * * * * * * * * * * * * * * * Destructor  * * * * * * * * * * * * * * * //


// * * * * * * * * * * * * * * * Member Functions  * * * * * * * * * * * * * //

void Foam::floatingBody::setMotion(tetPointVectorField& motionU)
{
    // Calculate force and moment
    const vector curForce = hullForce();
    const vector curMoment = hullMoment();

    // Grab initial force for the first time-step
    if (curTimeIndex_ == -1)
    {
        oldForce_ = curForce;
        oldMoment_ = curMoment;
        curTimeIndex_ = mesh_.time().timeIndex();
    }

    // Set force and moment
    sixDOF_.force().value() =
        0.5*(curForce + oldForce_)
      + sixDOF_.mass().value()*g_.value();

    sixDOF_.moment().value() =
        0.5*(curMoment + oldMoment_);


    // Manage multiple calls to mesh motion within a single time-step
    if (curTimeIndex_ < mesh_.time().timeIndex())
    {
        oldForce_ = curForce;
        oldMoment_ = curMoment;
        curTimeIndex_ = mesh_.time().timeIndex();
    }

    // Solve for motion
    solver_->solve
    (
        mesh_.time().value(),
        mesh_.time().value() +  mesh_.time().deltaT().value(),
        epsilon_,
        mesh_.time().deltaT().value()
    );

    Info<< "body: " << name() << nl
        << "f = " << sixDOF_.force().value() << tab
        << "mg = " << sixDOF_.mass().value()*g_.value() << tab
        << "M = " << sixDOF_.moment().value() << nl
        << "x = " << sixDOF_.X().value() << tab
        << "u = " << sixDOF_.Uaverage().value() << nl
        << "rot vector = " << sixDOF_.rotVector() << tab
        << "rot angle = " << sixDOF_.rotAngle().value() << tab
        << "omega = " << sixDOF_.omegaAverageAbsolute().value() << endl;

    // Set boundary motion
    fixedValueTetPolyPatchVectorField& motionUHull =
        refCast<fixedValueTetPolyPatchVectorField>
        (
            motionU.boundaryField()[hull_.index()]
        );

    // Set motion based on current hull geometry
    motionUHull == sixDOF_.boundaryVelocity(motionUHull.patch().localPoints());

    // Set motion on slider surfaces
    if (bargeSlider_.active())
    {
        fixedValueTetPolyPatchVectorField& motionUBargeSlider =
            refCast<fixedValueTetPolyPatchVectorField>
            (
                motionU.boundaryField()[bargeSlider_.index()]
            );

        motionUBargeSlider ==
            sixDOF_.boundaryVelocity(motionUBargeSlider.patch().localPoints());
    }

    if (fixedSlider_.active())
    {
        fixedValueTetPolyPatchVectorField& motionUFixedSlider =
            refCast<fixedValueTetPolyPatchVectorField>
            (
                motionU.boundaryField()[fixedSlider_.index()]
            );

        motionUFixedSlider == sixDOF_.Uaverage().value();
    }


    // Report position and rotation.  Write only if master
    if (Pstream::master())
    {
        of_ << mesh_.time().value() << tab
            << sixDOF_.X().value().x() << tab
            << sixDOF_.X().value().y() << tab
            << sixDOF_.X().value().z() << tab
            << sixDOF_.rotAngle().value() << endl;
    }
}


void Foam::floatingBody::writeDict(Ostream& os) const
{
    os  << nl << name() << nl << token::BEGIN_BLOCK;

    os  << "hull " << hull_.name() << token::END_STATEMENT << nl
        << "bargeSlider " << bargeSlider_.name() << token::END_STATEMENT << nl
        << "fixedSlider " << fixedSlider_.name() << token::END_STATEMENT << nl
        << "odeSolver" << solver_->type() << token::END_STATEMENT << nl
        << "epsilon" << epsilon_ << token::END_STATEMENT << nl
        << token::END_BLOCK << endl;
}


// ************************************************************************* //
